from abc import ABC, abstractmethod


class BaseEngineService(ABC):
    @abstractmethod
    def chat_completion(self, messages, ollama_config):
        pass
    
    @abstractmethod
    def embedding(self, messages, ollama_config):
        pass



import os

from made.chat_env.repository.chat_env_repository_impl import ChatEnvRepositoryImpl
from made.engine import ModelConfig
from made.phase import PhaseRegistry
from made.phase.entity.phase_chat_turn_limit import PhaseChatTurnLimit
from made.phase.entity.phase_engine_config import PhaseEngineConfig
from made.phase.entity.phase_prompts import PhasePrompt
from made.phase.repository.base_phase_repository_impl import BasePhaseRepositoryImpl
from made.role_playing.entity.role_prompts import RolePrompt
from made.role_playing.entity.role_type import RoleType
from made.tools.file.repository.file_tool_repository_impl import FileToolRepositoryImpl


@PhaseRegistry.register()
class DefaultCodeCompletePhaseRepositoryImpl(BasePhaseRepositoryImpl):
    def __init__(
        self,
        model_config: ModelConfig,
        phase_prompt: str = PhasePrompt.code_complete,
        assistant_role_name: str = RoleType.PROGRAMMER,
        assistant_role_prompt: str = RolePrompt.PROGRAMMER,
        user_role_name: str = RoleType.CTO,
        user_role_prompt: str = RolePrompt.CTO,
        chat_turn_limit: int = PhaseChatTurnLimit.code_complete,
    ):
        phase_engine_config = PhaseEngineConfig.code_complete
        super().__init__(
            model_config=model_config,
            phase_prompt=phase_prompt,
            assistant_role_name=assistant_role_name,
            assistant_role_prompt=assistant_role_prompt,
            user_role_name=user_role_name,
            user_role_prompt=user_role_prompt,
            chat_turn_limit=chat_turn_limit,
            temperature=phase_engine_config.temperature,
            top_p=phase_engine_config.top_p,
        )

    def update_phase_states(self, env: ChatEnvRepositoryImpl):
        self.states.task = env.config.task_prompt
        self.states.modality = env.states.modality
        self.states.language = env.states.language
        self.states.codes = env.states.codes

    def update_env_states(self, env: ChatEnvRepositoryImpl) -> ChatEnvRepositoryImpl:
        available_extensions = ["py", "js", "cpp", "c", "h", "java"]
        all_files = [
            file
            for file in os.listdir(env.config.directory)
            if file.split(".")[-1] in available_extensions
        ]
        for file_name in all_files:
            self.states.unimplemented_file = (
                file_name
                + "\n"
                + "```"
                + file_name.split(".")[-1]
                + "\n"
                + self.states.codes[file_name]
                + "\n```"
            )
            implemented_file = self.chatting(
                env=env,
                task_prompt=env.config.task_prompt,
                phase_prompt=self.phase_prompt,
                assistant_role_name=self.assistant_role_name,
                assistant_role_prompt=self.assistant_role_prompt,
                user_role_name=self.user_role_name,
                user_role_prompt=self.user_role_prompt,
                placeholders=self.states,
                chat_turn_limit=self.chat_turn_limit,
            )

            content = FileToolRepositoryImpl.abstract_contents_from_text(
                implemented_file,
                regex=r"(.+?)\n```.*?\n(.*?)```",
            )
            self.states.codes[file_name] = content[file_name]

            FileToolRepositoryImpl.write_file(
                os.path.join(env.config.directory, file_name), content[file_name]
            )

            # TODO git tool should be used below.

        env.states.codes = self.states.codes
        return env

    def execute(self, env):
        self.update_phase_states(env)
        env = self.update_env_states(env)
        return env



import os
from copy import deepcopy
from typing import Dict, Optional, Union

from made.agent.service.agent_service_impl import AgentServiceImpl
from made.chat_env.repository.chat_env_repository_impl import ChatEnvRepositoryImpl
from made.engine import ModelConfig
from made.messages.entity.chat_message.user_message import UserChatMessage
from made.messages.entity.system_message.assistant_system_message import (
    AssistantSystemMessage,
)
from made.phase.entity.phase_states import PhaseStates
from made.phase.repository.base_phase_repository import BasePhaseRepository
from made.role_playing.service.role_playing_service_impl import RolePlayingServiceImpl
from made.tools.rag.repository.rag_tool_repository_impl import RAGToolRepositoryImpl
from made.utils.logger import Logger


class BasePhaseRepositoryImpl(BasePhaseRepository):
    def __init__(
        self,
        model_config: ModelConfig,
        phase_prompt: str,
        assistant_role_name: str,
        assistant_role_prompt: str,
        user_role_name: str,
        user_role_prompt: str,
        chat_turn_limit: int = 10,
        conversation_rag: bool = False,
        conversation_logging: bool = True,
        **kwargs,
    ):
        model_config = deepcopy(model_config)
        self.model_config = model_config
        if temperature := kwargs.get("temperature"):
            if temperature is not None:
                self.model_config.temperature = temperature
        if top_p := kwargs.get("top_p"):
            if top_p is not None:
                self.model_config.top_p = top_p
        self.phase_prompt = phase_prompt
        self.assistant_role_name = assistant_role_name
        self.assistant_role_prompt = assistant_role_prompt
        self.user_role_name = user_role_name
        self.user_role_prompt = user_role_prompt
        self.chat_turn_limit = chat_turn_limit
        self.conversation_rag = conversation_rag
        self.conversation_logging = conversation_logging

        self.seminar_conclusion = None

        states = kwargs.get("states")
        self.states: Optional[Union[PhaseStates, Dict]] = (
            states if states else PhaseStates()
        )

    def chatting(
        self,
        env: ChatEnvRepositoryImpl,
        task_prompt: str,
        phase_prompt: str,
        assistant_role_name: str,
        assistant_role_prompt: str,
        user_role_name: str,
        user_role_prompt: str,
        placeholders=None,
        chat_turn_limit=10,
    ) -> str:
        if placeholders is None:
            placeholders = {}
        if not isinstance(placeholders, dict):
            placeholders = placeholders.__dict__
        assert 1 <= chat_turn_limit <= 100

        conversation_logger = None
        if self.conversation_logging:
            conversation_logger = Logger(
                f"{__class__.__name__}_{env.config.directory}",
                os.path.join(
                    env.config.directory, "logs", f"{ self.__class__.__name__}.log"
                ),
            ).get_logger()

        role_play_session = RolePlayingServiceImpl(
            model_config=self.model_config,
            task_prompt=task_prompt,
            assistant_role_name=assistant_role_name,
            assistant_role_prompt=assistant_role_prompt,
            user_role_name=user_role_name,
            user_role_prompt=user_role_prompt,
            background_prompt=env.config.background_prompt,
        )

        _, input_user_message = role_play_session.role_playing_repository.init_chat(
            placeholders, phase_prompt
        )
        # print()
        # print(
        #     f"\033[93m{input_user_message.role_name}\033[0m",
        #     ": ",
        #     input_user_message.content,
        # )
        if conversation_logger is not None:
            conversation_logger.info(
                f"{input_user_message.role_name}: {input_user_message.content}"
            )
        seminar_conclusion = None

        for _ in range(chat_turn_limit):
            assistant_response, user_response = role_play_session.step(
                input_user_message, chat_turn_limit == 1
            )
            # print()
            # print(
            #     f"\033[93m{assistant_response.message.role_name}\033[0m",
            #     ": ",
            #     assistant_response.message.content,
            # )
            if conversation_logger is not None:
                conversation_logger.info(
                    f"{assistant_response.message.role_name}: {assistant_response.message.content}"
                )
            if user_response.message is not None:
                # print()
                # print(
                #     f"\033[93m{user_response.message.role_name}\033[0m",
                #     ": ",
                #     user_response.message.content,
                # )
                if conversation_logger is not None:
                    conversation_logger.info(
                        f"{user_response.message.role_name}: {user_response.message.content}"
                    )
            if (
                role_play_session.role_playing_repository.assistant_agent.agent_repository.info
            ):
                seminar_conclusion = assistant_response.message.content
                break
            if assistant_response.terminated:
                break

            if (
                role_play_session.role_playing_repository.user_agent.agent_repository.info
            ):
                seminar_conclusion = user_response.message.content
                break
            if user_response.terminated:
                break

            if chat_turn_limit > 1:
                input_user_message = user_response.message
            else:
                break

        # TODO seminar conclusion should be more clear
        if seminar_conclusion is None:
            if self.conversation_rag:
                conversations = (
                    role_play_session.role_playing_repository.assistant_agent.agent_repository.stored_messages
                )
                task = conversations[0].content
                conversations = [
                    conversation.content for conversation in conversations[1:]
                ]
                client = RAGToolRepositoryImpl.get_client(
                    os.path.join(env.config.directory, "vectordb")
                )
                collection = RAGToolRepositoryImpl.get_collection(
                    client, "conversations"
                )
                RAGToolRepositoryImpl.ingest(
                    collection, conversations, self.model_config
                )
                search_result = RAGToolRepositoryImpl.search(
                    task, collection, self.model_config, n_results=2
                )
                rag_prompt = (
                    f"Query search result: {search_result}\n"
                    + f"Using the result, do {task}."
                )
                message = UserChatMessage(content=task)
                agent = AgentServiceImpl(
                    system_message=AssistantSystemMessage(
                        role_name=assistant_role_name, content=rag_prompt
                    ),
                    model_config=self.model_config,
                )
                seminar_conclusion = agent.step(message).message.content
                if "<INFO>" not in seminar_conclusion:
                    seminar_conclusion = "<INFO>" + seminar_conclusion
            else:
                seminar_conclusion = "<INFO>" + assistant_response.message.content

        seminar_conclusion = seminar_conclusion.split("<INFO>")[-1]
        # print()
        # print(f"\033[31mseminar conclusion\033[0m: {seminar_conclusion}")
        if conversation_logger is not None:
            conversation_logger.info(f"[seminar conclusion]: {seminar_conclusion}")
        return seminar_conclusion

    def update_phase_states(self, env: ChatEnvRepositoryImpl):
        """
        Update the phase states from previous phases states(global env).

        Args:
            env (ChatEnvRepositoryImpl): The chat environment.
        """
        raise NotImplementedError

    def update_env_states(self, env: ChatEnvRepositoryImpl):
        """
        Update the environment states based on the seminar conclusion.

        Args:
            env (ChatEnvRepositoryImpl): The chat environment.
        """
        raise NotImplementedError

    def execute(
        self,
        env: ChatEnvRepositoryImpl,
    ) -> ChatEnvRepositoryImpl:
        self.update_phase_states(env)
        self.seminar_conclusion = self.chatting(
            env=env,
            task_prompt=env.config.task_prompt,
            phase_prompt=self.phase_prompt,
            assistant_role_name=self.assistant_role_name,
            assistant_role_prompt=self.assistant_role_prompt,
            user_role_name=self.user_role_name,
            user_role_prompt=self.user_role_prompt,
            placeholders=self.states,
            chat_turn_limit=self.chat_turn_limit,
        )
        env = self.update_env_states(env)
        return env



from typing import List

from made.chat_env.repository.chat_env_repository_impl import ChatEnvRepositoryImpl
from made.engine import ModelConfig
from made.phase.repository.base_composed_phase_repository import (
    BaseComposedPhaseRepository,
)
from made.phase.repository.base_phase_repository_impl import BasePhaseRepositoryImpl
from made.phase.service.phase_service_impl import PhaseServiceImpl
from made.phase.entity.phase_states import PhaseStates


class BaseComposedPhaseRepositoryImpl(BaseComposedPhaseRepository):
    def __init__(
        self, model_config: ModelConfig, phases: List[str], states, num_cycle: int = 1
    ):
        self.phase_service = PhaseServiceImpl(model_config)
        self.phases = [self.phase_service.get_phase(phase) for phase in phases]
        self.states = states if states else PhaseStates()
        self.num_cycle = num_cycle

    def update_phase_states(self, env: ChatEnvRepositoryImpl):
        raise NotImplementedError

    def update_env_states(self, env: ChatEnvRepositoryImpl):
        raise NotImplementedError

    def break_cycle(self, phase_states):
        raise NotImplementedError

    def execute(self, env: ChatEnvRepositoryImpl):
        self.update_phase_states(env)
        for _ in range(self.num_cycle):
            for phase in self.phases:
                if isinstance(phase, BasePhaseRepositoryImpl):
                    phase.states = self.states
                    phase.update_phase_states(env)
                    if self.break_cycle(phase.states):
                        return env

                    env = phase.execute(env)
                    if self.break_cycle(phase.states):
                        return env

                elif isinstance(phase, BaseComposedPhaseRepositoryImpl):
                    phase = self.phase_service.get_phase(phase.__class__.__name__)
                    env = phase.execute(env)

                else:
                    raise NotImplementedError
        env = self.update_env_states(env)
        return env



from typing import List

import chromadb
from chromadb.api import ClientAPI
from chromadb import Collection

from made.tools.rag.repository.rag_tool_repository import RAGToolRepository
from made.engine import ModelConfig
from made.engine.service.ollama.ollama_engine_service_impl import (
    OllamaEngineServiceImpl,
)


class RAGToolRepositoryImpl(RAGToolRepository):
    @staticmethod
    def get_client(persist_path: str) -> ClientAPI:
        client = chromadb.PersistentClient(path=persist_path)
        return client

    @staticmethod
    def get_collection(client: ClientAPI, collection_name: str) -> Collection:
        collection = client.get_or_create_collection(name=collection_name)
        return collection

    @staticmethod
    def ingest(
        collection: Collection, documents: List[str], model_config: ModelConfig
    ) -> None:
        engine = OllamaEngineServiceImpl.get_instance()
        for idx, document in enumerate(documents):
            embeddings = engine.embedding(document, model_config).data[0].embedding
            collection.add(
                ids=[str(idx)], embeddings=[embeddings], documents=[document]
            )

    @staticmethod
    def search(
        query: str,
        collection: Collection,
        model_config: ModelConfig,
        n_results: int = 3,
    ) -> List[str]:
        engine = OllamaEngineServiceImpl.get_instance()
        query_embeddings = engine.embedding(query, model_config).data[0].embedding
        search_results = collection.query(
            query_embeddings=query_embeddings, n_results=n_results
        )["documents"][0][:n_results]
        return search_results



from abc import ABC, abstractmethod


class BaseComposedPhaseRepository(ABC):
    @abstractmethod
    def update_phase_states(self, env):
        pass

    @abstractmethod
    def update_env_states(self, env):
        pass

    @abstractmethod
    def break_cycle(self, phase_states):
        pass



from abc import ABC, abstractmethod


class RAGToolRepository(ABC):
    @staticmethod
    @abstractmethod
    def get_client(client_path):
        pass

    @staticmethod
    @abstractmethod
    def get_collection(client, collection_name):
        pass

    @staticmethod
    @abstractmethod
    def ingest(collection, documents):
        pass

    @staticmethod
    @abstractmethod
    def search(query):
        pass
